/* $Id: gram.c,v 1.2 2005/06/17 18:10:55 igor Exp $ */

/*
 * Copyright (c) 2004 Igor Boehm <igor@bytelabs.org>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF MIND, USE, DATA OR PROFITS, WHETHER
 * IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
 * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <sys/types.h>

#include <errno.h>
#include <fcntl.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "fail.h"
#include "macros.h"
#include "pbutil.h"
#include "pkgobject.h"
#include "gram.h"


/**
 * linecount():
 *
 * Count the number of lines of our INDEX file so that we know
 * how much space we are going to need to our package array.
 */
size_t
linecount(char *file)
{
       char            buf[MAXBSIZE];
       char           *c;
       int             len, fd;
       size_t          linect;

       linect = 0;
       if ((fd = open(file, O_RDONLY, 0)) < 0) 
               pb_warning(strerror(errno));
       else {
               while ((len = read(fd, buf, MAXBSIZE)) > 0) {
                       for (c = buf; len--; ++c)
                               if (*c == '\n')
                                       linect++;
               }
               if (len == -1)
                       pb_warning(strerror(errno));
       }
       if (close(fd))
               pb_warning(strerror(errno));
       if (pb_get_verbose_level())
               pb_message("%d Ports available.", linect);
       return linect;
}

/**
 * yyerror():
 *
 */
int
yyerror(const char *fmt,...)
{
       va_list         ap;
       char           *nfmt;

       va_start(ap, fmt);
       if (asprintf(&nfmt, "%s: %s", INDEXFILE, fmt) == -1)
               pb_error(1, "yyerror asprintf");
       vfprintf(stderr, nfmt, ap);
       va_end(ap);
       FREE(nfmt);
       return (0);
}

